@@ -1,5 +1,6 @@
 import new
 from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter.executioncontext import report_error
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import TypeDef, make_weakref_descr
 from pypy.interpreter.baseobjspace import W_Root
@@ -20,8 +21,8 @@ def descr_classobj_new(space, w_subtype, w_name, w_bases, w_dict):
     if not space.isinstance_w(w_dict, space.w_dict):
         raise_type_err(space, 'bases', 'tuple', w_bases)
 
-    if not space.contains_w(w_dict, space.wrap("__doc__")):
-        space.setitem(w_dict, space.wrap("__doc__"), space.w_None)
+    if not space.contains_w(w_dict, space.newtext("__doc__")):
+        space.setitem(w_dict, space.newtext("__doc__"), space.w_None)
 
     # XXX missing: lengthy and obscure logic about "__module__"
 
@@ -38,8 +39,10 @@ def descr_classobj_new(space, w_subtype, w_name, w_bases, w_dict):
 
 
 class W_ClassObject(W_Root):
+    _immutable_fields_ = ['bases_w?[*]', 'w_dict?']
+
     def __init__(self, space, w_name, bases, w_dict):
-        self.name = space.str_w(w_name)
+        self.name = space.text_w(w_name)
         make_sure_not_resized(bases)
         self.bases_w = bases
         self.w_dict = w_dict
@@ -61,9 +64,9 @@ class W_ClassObject(W_Root):
         self.w_dict = w_dict
 
     def setname(self, space, w_newname):
-        if not space.isinstance_w(w_newname, space.w_str):
+        if not space.isinstance_w(w_newname, space.w_text):
             raise oefmt(space.w_TypeError, "__name__ must be a string object")
-        self.name = space.str_w(w_newname)
+        self.name = space.text_w(w_newname)
 
     def setbases(self, space, w_bases):
         if not space.isinstance_w(w_bases, space.w_tuple):
@@ -75,6 +78,7 @@ class W_ClassObject(W_Root):
                             "__bases__ items must be classes")
         self.bases_w = bases_w
 
+    @jit.unroll_safe
     def is_subclass_of(self, other):
         assert isinstance(other, W_ClassObject)
         if self is other:
@@ -99,13 +103,13 @@ class W_ClassObject(W_Root):
                 return w_result
         return None
 
-    @unwrap_spec(name=str)
+    @unwrap_spec(name='text')
     def descr_getattribute(self, space, name):
         if name and name[0] == "_":
             if name == "__dict__":
                 return self.w_dict
             elif name == "__name__":
-                return space.wrap(self.name)
+                return space.newtext(self.name)
             elif name == "__bases__":
                 return space.newtuple(self.bases_w)
         w_value = self.lookup(space, name)
@@ -119,7 +123,7 @@ class W_ClassObject(W_Root):
         return space.call_function(w_descr_get, w_value, space.w_None, self)
 
     def descr_setattr(self, space, w_attr, w_value):
-        name = space.str_w(w_attr)
+        name = space.text_w(w_attr)
         if name and name[0] == "_":
             if name == "__dict__":
                 self.setdict(space, w_value)
@@ -134,11 +138,11 @@ class W_ClassObject(W_Root):
                 if not self.has_user_del(space):
                     msg = ("a __del__ method added to an existing class will "
                            "only be called on instances made from now on")
-                    space.warn(space.wrap(msg), space.w_RuntimeWarning)
+                    space.warn(space.newtext(msg), space.w_RuntimeWarning)
         space.setitem(self.w_dict, w_attr, w_value)
 
     def descr_delattr(self, space, w_attr):
-        name = space.str_w(w_attr)
+        name = space.text_w(w_attr)
         if name in ("__dict__", "__name__", "__bases__"):
             raise oefmt(space.w_TypeError,
                         "cannot delete attribute '%s'", name)
@@ -157,9 +161,9 @@ class W_ClassObject(W_Root):
     def descr_str(self, space):
         mod = self.get_module_string(space)
         if mod == "?":
-            return space.wrap(self.name)
+            return space.newtext(self.name)
         else:
-            return space.wrap("%s.%s" % (mod, self.name))
+            return space.newtext("%s.%s" % (mod, self.name))
 
     def get_module_string(self, space):
         try:
@@ -168,8 +172,8 @@ class W_ClassObject(W_Root):
             if not e.match(space, space.w_AttributeError):
                 raise
             return "?"
-        if space.isinstance_w(w_mod, space.w_str):
-            return space.str_w(w_mod)
+        if space.isinstance_w(w_mod, space.w_text):
+            return space.text_w(w_mod)
         return "?"
 
     def __repr__(self):
@@ -313,7 +317,7 @@ class W_InstanceObject(W_Root):
         # This method ignores the instance dict and the __getattr__.
         # Returns None if not found.
         assert isinstance(name, str)
-        w_value = self.w_class.lookup(space, name)
+        w_value = jit.promote(self.w_class).lookup(space, name)
         if w_value is None:
             return None
         w_descr_get = space.lookup(w_value, '__get__')
@@ -334,7 +338,7 @@ class W_InstanceObject(W_Root):
         w_meth = self.getattr_from_class(space, '__getattr__')
         if w_meth is not None:
             try:
-                return space.call_function(w_meth, space.wrap(name))
+                return space.call_function(w_meth, space.newtext(name))
             except OperationError as e:
                 if not exc and e.match(space, space.w_AttributeError):
                     return None     # eat the AttributeError
@@ -347,7 +351,7 @@ class W_InstanceObject(W_Root):
         else:
             return None
 
-    @unwrap_spec(name=str)
+    @unwrap_spec(name='text')
     def descr_getattribute(self, space, name):
         if len(name) >= 8 and name[0] == '_':
             if name == "__dict__":
@@ -357,7 +361,7 @@ class W_InstanceObject(W_Root):
         return self.getattr(space, name)
 
     def descr_setattr(self, space, w_name, w_value):
-        name = space.str_w(w_name)
+        name = space.text_w(w_name)
         w_meth = self.getattr_from_class(space, '__setattr__')
         if name and name[0] == "_":
             if name == '__dict__':
@@ -371,14 +375,14 @@ class W_InstanceObject(W_Root):
                     and self.getdictvalue(space, '__del__') is None):
                     msg = ("a __del__ method added to an instance with no "
                            "__del__ in the class will not be called")
-                    space.warn(space.wrap(msg), space.w_RuntimeWarning)
+                    space.warn(space.newtext(msg), space.w_RuntimeWarning)
         if w_meth is not None:
             space.call_function(w_meth, w_name, w_value)
         else:
             self.setdictvalue(space, name, w_value)
 
     def descr_delattr(self, space, w_name):
-        name = space.str_w(w_name)
+        name = space.text_w(w_name)
         if name and name[0] == "_":
             if name == '__dict__':
                 # use setdict to raise the error
@@ -429,14 +433,14 @@ class W_InstanceObject(W_Root):
             if space.len_w(w_format_spec) > 0:
                 msg = ("object.__format__ with a non-empty format string is "
                        "deprecated")
-                space.warn(space.wrap(msg), space.w_PendingDeprecationWarning)
+                space.warn(space.newtext(msg), space.w_PendingDeprecationWarning)
             return space.format(w_as_str, w_format_spec)
 
     def descr_len(self, space):
         w_meth = self.getattr(space, '__len__')
         w_result = space.call_function(w_meth)
         if space.isinstance_w(w_result, space.w_int):
-            if space.is_true(space.lt(w_result, space.wrap(0))):
+            if space.is_true(space.lt(w_result, space.newint(0))):
                 raise oefmt(space.w_ValueError, "__len__() should return >= 0")
             return w_result
         raise oefmt(space.w_TypeError, "__len__() should return an int")
@@ -498,7 +502,7 @@ class W_InstanceObject(W_Root):
                 return space.w_True
         w_result = space.call_function(w_func)
         if space.isinstance_w(w_result, space.w_int):
-            if space.is_true(space.lt(w_result, space.wrap(0))):
+            if space.is_true(space.lt(w_result, space.newint(0))):
                 raise oefmt(space.w_ValueError,
                             "__nonzero__() should return >= 0")
             return w_result
@@ -523,10 +527,10 @@ class W_InstanceObject(W_Root):
                                     "__cmp__ must return int")
                     raise
                 if res > 0:
-                    return space.wrap(1)
+                    return space.newint(1)
                 if res < 0:
-                    return space.wrap(-1)
-                return space.wrap(0)
+                    return space.newint(-1)
+                return space.newint(0)
         if isinstance(w_b, W_InstanceObject):
             w_func = w_b.getattr(space, '__cmp__', False)
             if w_func is not None:
@@ -541,10 +545,10 @@ class W_InstanceObject(W_Root):
                                     "__cmp__ must return int")
                     raise
                 if res < 0:
-                    return space.wrap(1)
+                    return space.newint(1)
                 if res > 0:
-                    return space.wrap(-1)
-                return space.wrap(0)
+                    return space.newint(-1)
+                return space.newint(0)
         return space.w_NotImplemented
 
     def descr_hash(self, space):
@@ -555,7 +559,7 @@ class W_InstanceObject(W_Root):
             if w_eq is not None or w_cmp is not None:
                 raise oefmt(space.w_TypeError, "unhashable instance")
             else:
-                return space.wrap(compute_identity_hash(self))
+                return space.newint(compute_identity_hash(self))
         w_ret = space.call_function(w_func)
         if (not space.isinstance_w(w_ret, space.w_int) and
             not space.isinstance_w(w_ret, space.w_long)):
@@ -568,6 +572,9 @@ class W_InstanceObject(W_Root):
             return space.call_function(w_func)
 
         w_truncated = space.trunc(self)
+        if (space.isinstance_w(w_truncated, space.w_int) or
+            space.isinstance_w(w_truncated, space.w_long)):
+            return w_truncated
         # int() needs to return an int
         try:
             return space.int(w_truncated)
@@ -591,7 +598,7 @@ class W_InstanceObject(W_Root):
     def descr_contains(self, space, w_obj):
         w_func = self.getattr(space, '__contains__', False)
         if w_func is not None:
-            return space.wrap(space.is_true(space.call_function(w_func, w_obj)))
+            return space.newbool(space.is_true(space.call_function(w_func, w_obj)))
         # now do it ourselves
         w_iter = space.iter(self)
         while 1:
@@ -650,9 +657,13 @@ class W_InstanceObject(W_Root):
         if w_func is None:
             w_func = self.getattr_from_class(space, '__del__')
         if w_func is not None:
-            if self.space.user_del_action.gc_disabled(self):
+            if space.user_del_action.gc_disabled(self):
                 return
-            space.call_function(w_func)
+            try:
+                space.call_function(w_func)
+            except Exception as e:
+                # report this came from __del__ vs a generic finalizer
+                report_error(space, e, "method __del__ of ", self)
 
     def descr_exit(self, space, w_type, w_value, w_tb):
         w_func = self.getattr(space, '__exit__', False)
