@@ -10,7 +10,18 @@ def test_skip_whitespace():
     assert dec.skip_whitespace(8) == len(s)
     dec.close()
 
-    
+def test_decode_key():
+    s1 = "123" * 100
+    s = ' "%s"   "%s" ' % (s1, s1)
+    dec = JSONDecoder('fake space', s)
+    assert dec.pos == 0
+    x = dec.decode_key(0)
+    assert x == s1
+    # check caching
+    y = dec.decode_key(dec.pos)
+    assert y == s1
+    assert y is x
+    dec.close()
 
 class AppTest(object):
     spaceconfig = {"objspace.usemodules._pypyjson": True}
@@ -190,6 +201,12 @@ class AppTest(object):
         res = _pypyjson.loads(json)
         assert res == {u'a': u'\ud83d'}
 
+    def test_cache_keys(self):
+        import _pypyjson
+        json = '[{"a": 1}, {"a": 2}]'
+        res = _pypyjson.loads(json)
+        assert res == [{u'a': 1}, {u'a': 2}]
+
     def test_tab_in_string_should_fail(self):
         import _pypyjson
         # http://json.org/JSON_checker/test/fail25.json
@@ -226,7 +243,7 @@ class AppTest(object):
             ('{"spam":[42}', "Unexpected '}' when decoding array (char 11)"),
             ('["]', 'Unterminated string starting at char 1'),
             ('["spam":', "Unexpected ':' when decoding array (char 7)"),
-            ('[{]', "No JSON object could be decoded: unexpected ']' at char 2"),
+            ('[{]', "Key name must be string at char 2"),
         ]
         for inputtext, errmsg in test_cases:
             exc = raises(ValueError, _pypyjson.loads, inputtext)
