@@ -119,18 +119,6 @@ class CConfigure:
         locals()[name] = rffi_platform.ConstantInteger(name)
     for name in xml_model_list:
         locals()[name] = rffi_platform.ConstantInteger(name)
-    for name in xml_model_list:
-        locals()[name] = rffi_platform.ConstantInteger(name)
-    for name in xml_model_list:
-        locals()[name] = rffi_platform.ConstantInteger(name)
-    for name in xml_model_list:
-        locals()[name] = rffi_platform.ConstantInteger(name)
-    for name in xml_model_list:
-        locals()[name] = rffi_platform.ConstantInteger(name)
-    for name in xml_model_list:
-        locals()[name] = rffi_platform.ConstantInteger(name)
-    for name in xml_model_list:
-        locals()[name] = rffi_platform.ConstantInteger(name)
     XML_Parser_SIZE = rffi_platform.SizeOf("XML_Parser")
 
 for k, v in rffi_platform.configure(CConfigure).items():
@@ -820,7 +808,7 @@ Return a new XML parser object."""
                     w_encoding)
 
     if space.is_none(w_namespace_separator):
-        namespace_separator = 0
+        namespace_separator = -1
     elif space.isinstance_w(w_namespace_separator, space.w_text):
         separator = space.text_w(w_namespace_separator)
         if len(separator) == 0:
@@ -843,7 +831,7 @@ Return a new XML parser object."""
     elif space.is_w(w_intern, space.w_None):
         w_intern = None
 
-    if namespace_separator:
+    if namespace_separator >= 0:
         xmlparser = XML_ParserCreateNS(
             encoding,
             rffi.cast(rffi.CHAR, namespace_separator))
