@@ -19,8 +19,11 @@ class State:
 
     def setinitialpath(self, space):
         from pypy.module.sys.initpath import compute_stdlib_path
-        # Initialize the default path
+        # This initial value for sys.prefix is normally overwritten
+        # at runtime by initpath.py
         srcdir = os.path.dirname(pypydir)
+        self.w_initial_prefix = space.newtext(srcdir)
+        # Initialize the default path
         path = compute_stdlib_path(self, srcdir)
         self.w_path = space.newlist([space.newtext(p) for p in path])
 
