@@ -478,7 +478,7 @@ class TestFunctions(BaseCTypesTestChecker):
         raises(ArgumentError, lambda: callback((1, 2, 3, 4), POINT()))
 
     def test_argument_conversion_and_checks(self):
-        py.test.skip("XXX currently broken on PyPy, sorry")
+        #This test is designed to check for segfaults if the wrong type of argument is passed as parameter
         strlen = dll.my_strchr
         strlen.argtypes = [c_char_p, c_int]
         strlen.restype = c_char_p
@@ -596,3 +596,37 @@ class TestFunctions(BaseCTypesTestChecker):
 
         get_data.errcheck = ret_list_p(1)
         assert get_data('testing!') == [-1, -2, -3, -4]
+
+    def test_issue2533(self):
+        import cffi
+        ffi = cffi.FFI()
+        ffi.cdef("int **fetchme(void);")
+        ffi.set_source("_x_cffi", """
+            int **fetchme(void)
+            {
+                static int a = 42;
+                static int *pa = &a;
+                return &pa;
+            }
+        """)
+        from rpython.tool.udir import udir
+        ffi.compile(verbose=True, tmpdir=str(udir))
+
+        import sys
+        sys.path.insert(0, str(udir))
+        try:
+            from _x_cffi import ffi, lib
+        finally:
+            sys.path.pop(0)
+        fetchme = ffi.addressof(lib, 'fetchme')
+        fetchme = int(ffi.cast("intptr_t", fetchme))
+
+        FN = CFUNCTYPE(POINTER(POINTER(c_int)))
+        ff = cast(fetchme, FN)
+
+        g = ff()
+        assert g.contents.contents.value == 42
+
+        h = c_int(43)
+        g[0] = pointer(h)     # used to crash here
+        assert g.contents.contents.value == 43
