@@ -143,7 +143,7 @@ def unpackcomplex(space, w_complex, strict_typing=True):
     w_z = None
     if space.is_oldstyle_instance(w_complex):
         try:
-            w_method = space.getattr(w_complex, space.wrap('__complex__'))
+            w_method = space.getattr(w_complex, space.newtext('__complex__'))
         except OperationError as e:
             if not e.match(space, space.w_AttributeError):
                 raise
@@ -170,12 +170,12 @@ def unpackcomplex(space, w_complex, strict_typing=True):
     # no '__complex__' method, so we assume it is a float,
     # unless it is an instance of some subclass of complex.
     if space.isinstance_w(w_complex, space.gettypefor(W_ComplexObject)):
-        real = space.float(space.getattr(w_complex, space.wrap("real")))
-        imag = space.float(space.getattr(w_complex, space.wrap("imag")))
+        real = space.float(space.getattr(w_complex, space.newtext("real")))
+        imag = space.float(space.getattr(w_complex, space.newtext("imag")))
         return (space.float_w(real), space.float_w(imag))
     #
     # Check that it is not a string (on which space.float() would succeed).
-    if (space.isinstance_w(w_complex, space.w_str) or
+    if (space.isinstance_w(w_complex, space.w_bytes) or
         space.isinstance_w(w_complex, space.w_unicode)):
         raise oefmt(space.w_TypeError,
                     "complex number expected, got '%T'", w_complex)
@@ -214,7 +214,7 @@ class W_ComplexObject(W_Root):
         return W_ComplexObject(rr, ir)
 
     def divmod(self, space, other):
-        space.warn(space.wrap("complex divmod(), // and % are deprecated"),
+        space.warn(space.newtext("complex divmod(), // and % are deprecated"),
                    space.w_DeprecationWarning)
         w_div = self.div(other)
         div = math.floor(w_div.realval)
@@ -251,10 +251,10 @@ class W_ComplexObject(W_Root):
             return False
         if self.user_overridden_class or w_other.user_overridden_class:
             return self is w_other
-        real1 = space.float_w(space.getattr(self, space.wrap("real")))
-        real2 = space.float_w(space.getattr(w_other, space.wrap("real")))
-        imag1 = space.float_w(space.getattr(self, space.wrap("imag")))
-        imag2 = space.float_w(space.getattr(w_other, space.wrap("imag")))
+        real1 = space.float_w(space.getattr(self, space.newtext("real")))
+        real2 = space.float_w(space.getattr(w_other, space.newtext("real")))
+        imag1 = space.float_w(space.getattr(self, space.newtext("imag")))
+        imag2 = space.float_w(space.getattr(w_other, space.newtext("imag")))
         real1 = float2longlong(real1)
         real2 = float2longlong(real2)
         imag1 = float2longlong(imag1)
@@ -267,8 +267,8 @@ class W_ComplexObject(W_Root):
         from rpython.rlib.longlong2float import float2longlong
         from pypy.objspace.std.util import IDTAG_COMPLEX as tag
         from pypy.objspace.std.util import IDTAG_SHIFT
-        real = space.float_w(space.getattr(self, space.wrap("real")))
-        imag = space.float_w(space.getattr(self, space.wrap("imag")))
+        real = space.float_w(space.getattr(self, space.newtext("real")))
+        imag = space.float_w(space.getattr(self, space.newtext("imag")))
         real_b = rbigint.fromrarith_int(float2longlong(real))
         imag_b = rbigint.fromrarith_int(r_ulonglong(float2longlong(imag)))
         val = real_b.lshift(64).or_(imag_b).lshift(IDTAG_SHIFT).int_or_(tag)
@@ -299,14 +299,14 @@ class W_ComplexObject(W_Root):
             and space.is_w(space.type(w_real), space.w_complex)):
             return w_real
 
-        if space.isinstance_w(w_real, space.w_str) or \
+        if space.isinstance_w(w_real, space.w_bytes) or \
                 space.isinstance_w(w_real, space.w_unicode):
             # a string argument
             if not noarg2:
                 raise oefmt(space.w_TypeError, "complex() can't take second"
                                                " arg if first is a string")
             try:
-                realstr, imagstr = _split_complex(space.str_w(w_real))
+                realstr, imagstr = _split_complex(space.text_w(w_real))
             except ValueError:
                 raise oefmt(space.w_ValueError,
                             "complex() arg is a malformed string")
@@ -347,25 +347,26 @@ class W_ComplexObject(W_Root):
 
     def descr_repr(self, space):
         if self.realval == 0 and copysign(1., self.realval) == 1.:
-            return space.wrap(repr_format(self.imagval) + 'j')
+            return space.newtext(repr_format(self.imagval) + 'j')
         sign = (copysign(1., self.imagval) == 1. or
                 isnan(self.imagval)) and '+' or ''
-        return space.wrap('(' + repr_format(self.realval)
-                          + sign + repr_format(self.imagval) + 'j)')
+        return space.newtext('(' + repr_format(self.realval)
+                             + sign + repr_format(self.imagval) + 'j)')
 
     def descr_str(self, space):
         if self.realval == 0 and copysign(1., self.realval) == 1.:
-            return space.wrap(str_format(self.imagval) + 'j')
+            return space.newtext(str_format(self.imagval) + 'j')
         sign = (copysign(1., self.imagval) == 1. or
                 isnan(self.imagval)) and '+' or ''
-        return space.wrap('(' + str_format(self.realval)
-                          + sign + str_format(self.imagval) + 'j)')
+        return space.newtext('(' + str_format(self.realval)
+                             + sign + str_format(self.imagval) + 'j)')
 
     def descr_hash(self, space):
         hashreal = _hash_float(space, self.realval)
-        hashimg = _hash_float(space, self.imagval)
-        combined = intmask(hashreal + 1000003 * hashimg)
-        return space.newint(combined)
+        hashimg = _hash_float(space, self.imagval)   # 0 if self.imagval == 0
+        h = intmask(hashreal + 1000003 * hashimg)
+        h -= (h == -1)
+        return space.newint(h)
 
     def descr_coerce(self, space, w_other):
         w_other = self._to_complex(space, w_other)
@@ -393,7 +394,7 @@ class W_ComplexObject(W_Root):
         try:
             return space.newfloat(math.hypot(self.realval, self.imagval))
         except OverflowError as e:
-            raise OperationError(space.w_OverflowError, space.wrap(str(e)))
+            raise OperationError(space.w_OverflowError, space.newtext(str(e)))
 
     def descr_eq(self, space, w_other):
         if isinstance(w_other, W_ComplexObject):
@@ -475,7 +476,7 @@ class W_ComplexObject(W_Root):
         try:
             return self.div(w_rhs)
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
 
     def descr_rtruediv(self, space, w_lhs):
         w_lhs = self._to_complex(space, w_lhs)
@@ -484,7 +485,7 @@ class W_ComplexObject(W_Root):
         try:
             return w_lhs.div(self)
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
 
     def descr_floordiv(self, space, w_rhs):
         w_rhs = self._to_complex(space, w_rhs)
@@ -494,7 +495,7 @@ class W_ComplexObject(W_Root):
         try:
             return self.divmod(space, w_rhs)[0]
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
 
     def descr_rfloordiv(self, space, w_lhs):
         w_lhs = self._to_complex(space, w_lhs)
@@ -504,7 +505,7 @@ class W_ComplexObject(W_Root):
         try:
             return w_lhs.divmod(space, self)[0]
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
 
     def descr_mod(self, space, w_rhs):
         w_rhs = self._to_complex(space, w_rhs)
@@ -513,7 +514,7 @@ class W_ComplexObject(W_Root):
         try:
             return self.divmod(space, w_rhs)[1]
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
 
     def descr_rmod(self, space, w_lhs):
         w_lhs = self._to_complex(space, w_lhs)
@@ -522,7 +523,7 @@ class W_ComplexObject(W_Root):
         try:
             return w_lhs.divmod(space, self)[1]
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
 
     def descr_divmod(self, space, w_rhs):
         w_rhs = self._to_complex(space, w_rhs)
@@ -531,7 +532,7 @@ class W_ComplexObject(W_Root):
         try:
             div, mod = self.divmod(space, w_rhs)
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
         return space.newtuple([div, mod])
 
     def descr_rdivmod(self, space, w_lhs):
@@ -541,7 +542,7 @@ class W_ComplexObject(W_Root):
         try:
             div, mod = w_lhs.divmod(space, self)
         except ZeroDivisionError as e:
-            raise OperationError(space.w_ZeroDivisionError, space.wrap(str(e)))
+            raise OperationError(space.w_ZeroDivisionError, space.newtext(str(e)))
         return space.newtuple([div, mod])
 
     @unwrap_spec(w_third_arg=WrappedDefault(None))
