@@ -728,6 +728,17 @@ def op_gc_load_indexed(TVAL, p, index, scale, base_ofs):
     return p[0]
 op_gc_load_indexed.need_result_type = True
 
+def op_gc_store_indexed(p, index, newvalue, scale, base_ofs):
+    # 'base_ofs' should be a CompositeOffset(..., ArrayItemsOffset).
+    # 'scale' should be a llmemory.sizeof().
+    from rpython.rtyper.lltypesystem import rffi
+    TVAL = lltype.typeOf(newvalue)
+    ofs = base_ofs + scale * index
+    if isinstance(ofs, int):
+        return op_raw_store(p, ofs, newvalue)
+    p = rffi.cast(rffi.CArrayPtr(TVAL), llmemory.cast_ptr_to_adr(p) + ofs)
+    p[0] = newvalue
+
 def op_likely(x):
     assert isinstance(x, bool)
     return x
