@@ -1005,6 +1005,16 @@ class ListComprehensionDetector(object):
 
         # - add a hint(vlist, iterable, {'maxlength'}) in the iterblock,
         #   where we can compute the known maximum length
+        # - new in June 2017: we do that only if 'exactlength' is True.
+        #   found some real use cases where the over-allocation scheme
+        #   was over-allocating far too much: the loop would only append
+        #   an item to the list after 'if some rare condition:'.  By
+        #   dropping this hint, we disable preallocation and cause the
+        #   append() to be done by checking the size, but still, after
+        #   the loop, we will turn the list into a fixed-size one.
+        #   ('maxlength_inexact' is never processed elsewhere; the hint
+        #   is still needed to prevent this function from being entered
+        #   in an infinite loop)
         link = iterblock.exits[0]
         vlist = self.contains_vlist(link.args)
         assert vlist
@@ -1014,7 +1024,8 @@ class ListComprehensionDetector(object):
                 break
         else:
             raise AssertionError("lost 'iter' operation")
-        chint = Constant({'maxlength': True})
+        chint = Constant({'maxlength' if exactlength else 'maxlength_inexact':
+                          True})
         hint = op.hint(vlist, hlop.args[0], chint)
         iterblock.operations.append(hint)
         link.args = list(link.args)
